#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import re
import subprocess


if sys.version_info < (3, ):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True

#add lib path
playbook_file = os.path.realpath(sys.argv[0])
playbook_path = os.path.split(playbook_file)[0]

lib_path = os.path.join(playbook_path, 'library')
if os.path.exists(lib_path) is True:
    sys.path.insert(0, lib_path)
    lib_path = os.path.join(lib_path, 'securecat')
    if os.path.exists(lib_path) is True:
        sys.path.insert(0, lib_path)
        modulepath = os.environ.get('CSC_MODULE_PATH')
        if modulepath is None:
            os.environ['CSC_MODULE_PATH'] = lib_path

from utils import playbook


class MyPlaybook(playbook.Playbook):
    def set_extra_vars_pre(self):
        self._runcontext.set_extra_var('PID','')
        
    def run_cmd(self, cmd):
        p = subprocess.Popen(cmd, env=os.environ, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE)
        self._runcontext.sleep()
        output = p.stdout.read().strip()
        p.stdout.close()

        if PY3:
            output = output.decode('utf-8')

        return output

    def set_extra_vars_post(self):
        var_vsftpd_pid = self._runcontext.get_extra_var('PID')
        var_vsftpd_pid = self._runcontext.replace_inject_characters(str(var_vsftpd_pid))
        if var_vsftpd_pid == "":
            raise Exception('Must specify extra variable PID.')
        
            
        vsftpd_cmd = 'ps -wp %s -o cmd | grep -iv cmd' % var_vsftpd_pid
        self._runcontext.log_debug('vsftpd command = %s' % vsftpd_cmd)
        
        vsftpd_cmd = self.run_cmd(vsftpd_cmd)
        m = re.match(r'.+\s(\w+\.conf)', vsftpd_cmd)
        vsftpd_cnf = ''
        if m is not None:
            vsftpd_cnf = m.groups()[0]
            vsftpd_cnf_tmp = os.path.realpath(vsftpd_cnf)
            if not os.path.exists(vsftpd_cnf_tmp):
                vsftpd_cwd = '/proc/%s/cwd' % var_vsftpd_pid
                vsftpd_cwd = os.readlink(vsftpd_cwd)
                vsftpd_cnf_tmp = os.path.join(vsftpd_cwd, vsftpd_cnf)
                if not os.path.exists(vsftpd_cnf_tmp):
                    raise Exception('vsftpd config file is not find.')
                vsftpd_cnf = vsftpd_cnf_tmp
            else:
                vsftpd_cnf = vsftpd_cnf_tmp
        else:
            vsftpd_cnf = "/etc/vsftpd.conf"
            if not os.path.exists(vsftpd_cnf):
                vsftpd_cnf = "/etc/vsftpd/vsftpd.conf"
                if not os.path.exists(vsftpd_cnf):
                    raise Exception('vsftpd config file is not find.')
        
        vsftpd_cnf = self._runcontext.replace_inject_characters(vsftpd_cnf)        
        self._runcontext.set_extra_var('VSFTPD_CONFIG_FILE', vsftpd_cnf)
        
    def get_tasks(self, dir):
        ret = []
        ret.append([dir, 'r_1_1.py'])
        ret.append([dir, 'r_1_2.py'])
        ret.append([dir, 'r_1_3.py'])
        ret.append([dir, 'r_1_4.py'])
        ret.append([dir, 'r_1_5.py'])
        ret.append([dir, 'r_1_6.py'])
        ret.append([dir, 'r_1_7.py'])
        ret.append([dir, 'r_1_8.py'])

        return ret


if __name__ == '__main__':

    myplaybook = MyPlaybook()
    myplaybook.set_role('hw')
    myplaybook.run()
