# -*- coding: UTF-8 -*-
import re
import json


def get_tags():
    return ['1', '1.2']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)

    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):
    curtemplate = runcontext.get_template()
    vsftpd_config_file = runcontext.get_extra_var('VSFTPD_CONFIG_FILE')
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["scancommand1"] = r'grep -Ev "^\s*#|^\s*$" %s | grep userlist_enable' % vsftpd_config_file
    tmp_vars["systemvalue1"] = ""
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["scancommand2"] = r'grep -Ev "^\s*#|^\s*$" %s | grep userlist_deny' % vsftpd_config_file
    tmp_vars["systemvalue2"] = ""
    tmp_vars["checkresult3"] = "failed"
    tmp_vars["scancommand3"] = r'grep -Ev "^\s*#|^\s*$" %s 2>/dev/null | grep userlist_file' % vsftpd_config_file
    tmp_vars["systemvalue3"] = ""
    ret_code = -1

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)

    if arc1 != 0:
        return (-2, "Failed to invoke module: %s." % astdout1)

    resultjson1 = json.loads(astdout1)
    astdout1 = resultjson1["stdout"]
    astderr1 = resultjson1["stderr"]

    match1_found = re.match(r"\s*userlist_enable\s*=\s*yes", astdout1.lower())
    if match1_found is None:
        tmp_vars["systemvalue1"] = astdout1
    else:
        tmp_vars["checkresult1"] = "pass"
        tmp_vars["systemvalue1"] = astdout1

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
    arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)

    if arc2 != 0:
        return (-2, "Failed to invoke module: %s." % astdout2)

    resultjson2 = json.loads(astdout2)
    astdout2 = resultjson2["stdout"]
    astderr2 = resultjson2["stderr"]

    match2_found = re.match(r"\s*userlist_deny\s*=\s*no", astdout2.lower())
    if match2_found is None:
        tmp_vars["systemvalue2"] = astdout2
    else:
        tmp_vars["checkresult2"] = "pass"
        tmp_vars["systemvalue2"] = astdout2

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand3"])
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc3, astdout3, astderr3)

    if arc3 != 0:
        return (-2, "Failed to invoke module: %s." % astdout3)

    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    astderr3 = resultjson3["stderr"]

    match3_found = re.match(".+", astdout3)
    if match3_found is None:
        tmp_vars["systemvalue3"] = astdout3
    else:
        tmp_vars["checkresult3"] = "pass"
        tmp_vars["systemvalue3"] = astdout3

    if tmp_vars["checkresult1"] == "pass" and tmp_vars["checkresult2"] == "pass" and tmp_vars["checkresult3"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_1_2.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
