# -*- coding: UTF-8 -*-
import re
import json


def get_tags():
    return ['1', '1.3']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)

    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):
    curtemplate = runcontext.get_template()
    vsftpd_config_file = runcontext.get_extra_var('VSFTPD_CONFIG_FILE')
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""
    cmd1 = r'grep -Ev "^\s*#|^\s*$" %s | grep chroot_local_user' % vsftpd_config_file
    cmd2 = r'grep -Ev "^\s*#|^\s*$" %s | grep chroot_list_enable' % vsftpd_config_file
    ret_code = -1

    runcontext.set_mod_params_instance(ansible_params, "command", cmd1)
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)

    if arc1 != 0:
        return (-2, "Failed to invoke module: %s." % astdout1)

    resultjson1 = json.loads(astdout1)
    astdout1 = resultjson1["stdout"]
    astderr1 = resultjson1["stderr"]

    runcontext.set_mod_params_instance(ansible_params, "command", cmd2)
    arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)

    if arc2 != 0:
        return (-2, "Failed to invoke module: %s." % astdout2)

    resultjson2 = json.loads(astdout2)
    astdout2 = resultjson2["stdout"]
    astderr2 = resultjson2["stderr"]

    tmp_vars["systemvalue1"] = "%s,%s" % (astdout1, astdout2)

    pattern1 = re.compile(r"chroot_local_user\s*=\s*yes")
    m1 = pattern1.findall(astdout1.lower())

    pattern2 = re.compile(r"chroot_list_enable\s*=\s*yes")
    m2 = pattern2.findall(astdout2.lower())

    if m1 or m2:
        tmp_vars["checkresult1"] = "pass"

    if tmp_vars["checkresult1"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_1_3.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
