# -*- coding: UTF-8 -*-
import re
import os
import pwd
import grp
import traceback
from stat import S_IMODE

def get_tags():
    return ['1', '1.6']


def get_usr_grp_mode(file_path):
    '''get user,group,mode'''
    stat_ste = os.stat(file_path)
    user = pwd.getpwuid(stat_ste.st_uid).pw_name
    group = grp.getgrgid(stat_ste.st_gid).gr_name
    filemode = "%04o" % S_IMODE(stat_ste.st_mode)
    return user, group, filemode


def change_mod_list(except_mode):
    def get_list(mode):
        x_list = []
        if mode // 4 == 1:
            x_list.append(4)
            mode = mode % 4
        if mode // 2 == 1:
            x_list.append(2)
            mode = mode % 2
        if mode // 1 == 1:
            x_list.append(1)
        return x_list

    if len(except_mode) == 4:
        except_mode = except_mode[1:]

    u_mod = int(except_mode[0])
    g_mod = int(except_mode[1])
    o_mod = int(except_mode[2])
    u_list = get_list(u_mod)
    g_list = get_list(g_mod)
    o_list = get_list(o_mod)

    return u_list, g_list, o_list


def compare_list(check_list, target_list):
    for x in check_list:
        if x not in target_list:
            return False
    return True


def compare(filemode, targetmode):
    u_list, g_list, o_list = change_mod_list(filemode)
    tu_list, tg_list, to_list = change_mod_list(targetmode)
    if compare_list(u_list, tu_list) and compare_list(g_list, tg_list) and compare_list(o_list, to_list):
        return True
    else:
        return False


def get_file(vsftpd_config_file):
    f = None
    file2 = ''
    file3 = ''
    try:
        f = open(vsftpd_config_file, 'r')
        lines = f.readlines()
        for line in lines:
            line = line.strip()
            if line.startswith('#') or not line:
                continue  
            m = re.match(r'(<param>.+)(<comment>#.*)', line)
            if m is not None:
                line = m.group('comment')
            if "chroot_list_file" in line:
                file2 = line.split('=')[1].strip()
            if "userlist_file" in line:
                file3 = line.split('=')[1].strip()
    except:
        traceback.format_exc()
    finally:
        if f is not None:
            f.close()

    if not file2:
        if not os.path.exists('/etc/vsftpd.chroot_list'):
            if not os.path.exists('/etc/vsftpd/vsftpd.chroot_list'):
                file2 = ""
            else:
                file2 = '/etc/vsftpd/vsftpd.chroot_list'
        else:
            file2 = '/etc/vsftpd.chroot_list'

    if not file3:
        if not os.path.exists('/etc/vsftpd.user_list'):
            if not os.path.exists('/etc/vsftpd/vsftpd.user_list'):
                file3 = ""
            else:
                file3 = '/etc/vsftpd/vsftpd.user_list'
        else:
            file3 = '/etc/vsftpd.user_list'

    return file2, file3

    
def run_task(runcontext):
    rc, out = run_audit(runcontext)
    return rc, out


def run_audit(runcontext):
    curtemplate = runcontext.get_template()
    vsftpd_config_file = runcontext.get_extra_var('VSFTPD_CONFIG_FILE')
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["systemvalue1"] = ""
    tmp_vars["checkresult2"] = "failed"
    tmp_vars["systemvalue2"] = ""
    tmp_vars["checkresult3"] = "failed"
    tmp_vars["systemvalue3"] = ""
    ret_code = -1

    file1 = vsftpd_config_file
    if os.path.exists(file1):
        user1, group1, filemode1 = get_usr_grp_mode(file1)
        tmp_vars["systemvalue1"] = "%s\t%s:%s\t%s" % (file1, user1, group1, filemode1)
        if user1 == 'root' and group1 == 'root' and compare(filemode1, '600'):
            tmp_vars["checkresult1"] = "pass"
    else:
        tmp_vars["systemvalue1"] = '%s is not exists.' % file1

    file2, file3 = get_file(vsftpd_config_file)

    if file2:
        if os.path.exists(file2):
            user2, group2, filemode2 = get_usr_grp_mode(file2)
            tmp_vars["systemvalue2"] = "%s\t%s:%s\t%s" % (file2, user2, group2, filemode2)
            if user2 == 'root' and group2 == 'root' and compare(filemode2, '640'):
                tmp_vars["checkresult2"] = "pass"
        else:
            tmp_vars["systemvalue2"] = '%s is not exists.' % file2
    else:
        tmp_vars["systemvalue2"] = 'vsftpd.chroot_list is not config.'
        tmp_vars["checkresult2"] = "pass"
    
    if file3:
        if os.path.exists(file3):
            user3, group3, filemode3 = get_usr_grp_mode(file3)
            tmp_vars["systemvalue3"] = "%s\t%s:%s\t%s" % (file3, user3, group3, filemode3)
            if user3 == 'root' and group3 == 'root' and compare(filemode3, '640'):
                tmp_vars["checkresult3"] = "pass"
        else:
            tmp_vars["systemvalue3"] = '%s is not exists.' % file3
    else:
        tmp_vars["systemvalue3"] = 'vsftpd.user_list is not config.'
        tmp_vars["checkresult3"] = "pass"

    if tmp_vars["checkresult1"] == "pass" and tmp_vars["checkresult2"] == "pass" and tmp_vars["checkresult3"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_1_6.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
