# -*- coding: UTF-8 -*-
import re
import json


def get_tags():
    return ['1', '1.7']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)

    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):
    curtemplate = runcontext.get_template()
    vsftpd_config_file = runcontext.get_extra_var('VSFTPD_CONFIG_FILE')
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["scancommand1"] = r'grep -Ev "^\s*#|^\s*$"  %s | grep ssl_enable' % vsftpd_config_file
    tmp_vars["systemvalue1"] = ""
    tmp_vars["checkresult2"] = "pass"
    tmp_vars["scancommand2"] = r'grep -v "^\s*#|^\s*$"  %s | grep allow_anon_ssl' % vsftpd_config_file
    tmp_vars["systemvalue2"] = ""
    tmp_vars["checkresult3"] = "pass"
    tmp_vars["scancommand3"] = r'grep -v "^\s*#|^\s*$"  %s | grep force_local_data_ssl' % vsftpd_config_file
    tmp_vars["systemvalue3"] = ""
    tmp_vars["checkresult4"] = "pass"
    tmp_vars["scancommand4"] = r'grep -v "^\s*#|^\s*$"  %s | grep force_local_logins_ssl' % vsftpd_config_file
    tmp_vars["systemvalue4"] = ""
    ret_code = -1

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)

    if arc1 != 0:
        return (-2, "Failed to invoke module: %s." % astdout1)
    resultjson1 = json.loads(astdout1)
    astdout1 = resultjson1["stdout"]
    astderr1 = resultjson1["stderr"]
    tmp_vars["systemvalue1"] = astdout1
    match1_found = re.match(r"\s*ssl_enable\s*=\s*yes", astdout1.lower())
    if match1_found is not None:
        tmp_vars["checkresult1"] = "pass"
        
    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
    arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)

    if arc2 != 0:
        return (-2, "Failed to invoke module: %s." % astdout2)
    resultjson2 = json.loads(astdout2)
    astdout2 = resultjson2["stdout"]
    astderr2 = resultjson2["stderr"]
    tmp_vars["systemvalue2"] = astdout2
    match2_found = re.match(r"\s*allow_anon_ssl\s*=\s*yes", astdout2.lower())
    if match2_found is not None:
        tmp_vars["checkresult2"] = "failed"
    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand3"])
    arc3, astdout3, astderr3 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc3, astdout3, astderr3)

    if arc3 != 0:
        return (-2, "Failed to invoke module: %s." % astdout3)
    resultjson3 = json.loads(astdout3)
    astdout3 = resultjson3["stdout"]
    astderr3 = resultjson3["stderr"]
    tmp_vars["systemvalue3"] = astdout3
    match3_found = re.match(r"\s*force_local_data_ssl\s*=\s*no", astdout3.lower())
    if match3_found is not None:
        tmp_vars["checkresult3"] = "failed"

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand4"])
    arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansible_params)
    runcontext.sleep()
    runcontext.log_debug_invoke_result(arc4, astdout4, astderr4)

    if arc4 != 0:
        return (-2, "Failed to invoke module: %s." % astdout4)
    resultjson4 = json.loads(astdout4)
    astdout4 = resultjson4["stdout"]
    astderr4 = resultjson4["stderr"]
    tmp_vars["systemvalue4"] = astdout4
    match4_found = re.match(r"\s*force_local_logins_ssl\s*=\s*no", astdout4.lower())
    if match4_found is not None:
        tmp_vars["checkresult4"] = "failed"

    if tmp_vars["checkresult1"] == "pass" and tmp_vars["checkresult2"] == "pass" and tmp_vars["checkresult3"] == "pass" and tmp_vars["checkresult4"] == "pass":
        ret_code = 0
        tmp_vars["policyresult"] = "pass"

    rc, out = curtemplate.render(runcontext.get_template_file('r_1_7.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
